#include <windows.h>
#include <airpcap.h>
#include <stdio.h>
#include <conio.h> //for _kbhit()

static
void ProcessPackets(PBYTE pBuffer, UINT receivedBytes);

#pragma pack(push, 1)
typedef struct _PPI_PACKET_HEADER
{
	UCHAR	PphVersion;
	UCHAR	PphFlags;
	USHORT	PphLength;
	ULONG	PphDlt;
}
	PPI_PACKET_HEADER, *PPPI_PACKET_HEADER;

#pragma pack(pop)

int main(int argc, char* argv[])
{
	CHAR errBuf[AIRPCAP_ERRBUF_SIZE];
	AirpcapDeviceDescription *pDevList, *pCursor;
	AirpcapChannelInfo channelInfo;
	PAirpcapHandle pAirPcap;
	ULONG numDevices;
	ULONG devIndex;
	ULONG i;
	PBYTE pBuffer;
	UINT bufferSize;
	UINT receivedBytes;
	HANDLE hReadEvent;

	if(AirpcapGetDeviceList(&pDevList, errBuf) == FALSE)
	{
		printf("Unable to get the list of devices: %s\n", errBuf);
		return -1;
	}

	numDevices = 0;
	
	for(pCursor = pDevList; pCursor != NULL; pCursor = pCursor->next)
	{
		printf("%u) %s (%s)\n", 
		numDevices,
		pCursor->Name,
		pCursor->Description);

		numDevices ++;
	}

	//
	// loop until the user chooses a valid device index
	//
	do
	{
		printf("Device (0 - %u)? ", numDevices - 1);
	}
	while(scanf("%u", &devIndex) == 0 || devIndex >= numDevices);

	//
	// pick the adapter from the list
	//
	for (pCursor = pDevList, i = 0; i != devIndex; pCursor = pCursor->next, i++);

	//
	// open the chosen device
	//
	pAirPcap = AirpcapOpen(pCursor->Name, errBuf);

	//
	// we do not need the list of devices any more, free it
	//
	AirpcapFreeDeviceList(pDevList);

	if (pAirPcap == NULL)
	{
		printf("Error opening device: %s\n", errBuf);
		return -1;
	}

	//
	// try to set the channel frequency
	//
	channelInfo.Flags = 0;
	channelInfo.Frequency = 2412; //channel 1
	channelInfo.ExtChannel = 0;
	channelInfo.Reserved[0] = 0;
	channelInfo.Reserved[1] = 0;

	if (AirpcapSetDeviceChannelEx(pAirPcap, channelInfo) == FALSE)
	{
		printf("Cannot set the channel: %s\n", AirpcapGetLastError(pAirPcap));
		AirpcapClose(pAirPcap);
		return -1;
	}

	//
	// set the encapsulation to PPI
	//
	if (AirpcapSetLinkType(pAirPcap, AIRPCAP_LT_802_11_PLUS_PPI) == FALSE)
	{
		printf("Cannot set the link type to PPI: %s\n", AirpcapGetLastError(pAirPcap));
		AirpcapClose(pAirPcap);
		return -1;
	}

	//
	// get a handle we can wait on for the reception of packets
	//
	if (AirpcapGetReadEvent(pAirPcap, &hReadEvent) == FALSE)
	{
		printf("Cannot get the read event: %s\n", AirpcapGetLastError(pAirPcap));
		AirpcapClose(pAirPcap);
		return -1;
	}

	//
	// allocate the rx buffer.
	// TODO check for malloc failure
	//
	bufferSize = 1024 * 1024;
	pBuffer = (PBYTE)malloc(bufferSize);

	//
	// receive the packets 
	//
	do
	{
		//
		// wait until some packets are available, or 500 ms
		//
		WaitForSingleObject(hReadEvent, 500);

		if (AirpcapRead(pAirPcap, pBuffer, bufferSize, &receivedBytes) == FALSE)
		{
			printf("Cannot read the packets: %s\n", AirpcapGetLastError(pAirPcap));
			break;
		}

		if (receivedBytes > 0)
		{
			ProcessPackets(pBuffer, receivedBytes);
		}

	}while(!_kbhit());

	AirpcapClose(pAirPcap);

	return 0;
}

void ProcessPackets(PBYTE pBuffer, UINT receivedBytes)
{
	UINT cursor = 0;
	PAirpcapBpfHeader pHeader;
	PBYTE pData;
	PPPI_PACKET_HEADER pPpiHeader;

	while(cursor < receivedBytes)
	{
		pHeader = (PAirpcapBpfHeader)(pBuffer + cursor);
		cursor += pHeader->Hdrlen;
		pData = pBuffer + cursor;
		cursor += AIRPCAP_WORDALIGN(pHeader->Caplen);
		
		//
		// get the PPI header length
		//
		pPpiHeader = (PPPI_PACKET_HEADER)pData;

		//
		// finally print the packet
		//
		printf("Received packet of size %u", pHeader->Originallen - pPpiHeader->PphLength);
	}
}